@@ -11,9 +11,10 @@ from pyshark.capture.live_capture import LiveCapture, UnknownInterfaceException
 from pyshark.packet.packet import Packet  # type: ignore
 
 from roborock import RoborockException
-from roborock.containers import DeviceData, LoginData
+from roborock.containers import DeviceData, HomeDataProduct, LoginData
 from roborock.protocol import MessageParser
 from roborock.util import run_sync
+from roborock.version_1_apis.roborock_local_client_v1 import RoborockLocalClientV1
 from roborock.version_1_apis.roborock_mqtt_client_v1 import RoborockMqttClientV1
 from roborock.web_api import RoborockApiClient
 
@@ -61,7 +62,11 @@ def cli(ctx, debug: int):
 
 @click.command()
 @click.option("--email", required=True)
-@click.option("--password", required=True)
+@click.option(
+    "--password",
+    required=False,
+    help="Password for the Roborock account. If not provided, an email code will be requested.",
+)
 @click.pass_context
 @run_sync()
 async def login(ctx, email, password):
@@ -74,7 +79,14 @@ async def login(ctx, email, password):
     except RoborockException:
         pass
     client = RoborockApiClient(email)
-    user_data = await client.pass_login(password)
+    if password is not None:
+        user_data = await client.pass_login(password)
+    else:
+        print(f"Requesting code for {email}")
+        await client.request_code()
+        code = click.prompt("A code has been sent to your email, please enter the code", type=str)
+        user_data = await client.code_login(code)
+        print("Login successful")
     context.update(LoginData(user_data=user_data, email=email))
 
 
@@ -113,6 +125,61 @@ async def list_devices(ctx):
     click.echo(f"Known devices {device_name_id}")
 
 
+@click.command()
+@click.option("--device_id", required=True)
+@click.pass_context
+@run_sync()
+async def list_scenes(ctx, device_id):
+    context: RoborockContext = ctx.obj
+    login_data = context.login_data()
+    if not login_data.home_data:
+        await _discover(ctx)
+        login_data = context.login_data()
+    client = RoborockApiClient(login_data.email)
+    scenes = await client.get_scenes(login_data.user_data, device_id)
+    output_list = []
+    for scene in scenes:
+        output_list.append(scene.as_dict())
+    click.echo(json.dumps(output_list, indent=4))
+
+
+@click.command()
+@click.option("--scene_id", required=True)
+@click.pass_context
+@run_sync()
+async def execute_scene(ctx, scene_id):
+    context: RoborockContext = ctx.obj
+    login_data = context.login_data()
+    if not login_data.home_data:
+        await _discover(ctx)
+        login_data = context.login_data()
+    client = RoborockApiClient(login_data.email)
+    await client.execute_scene(login_data.user_data, scene_id)
+
+
+@click.command()
+@click.option("--device_id", required=True)
+@click.pass_context
+@run_sync()
+async def status(ctx, device_id):
+    context: RoborockContext = ctx.obj
+    login_data = context.login_data()
+    if not login_data.home_data:
+        await _discover(ctx)
+        login_data = context.login_data()
+    home_data = login_data.home_data
+    devices = home_data.devices + home_data.received_devices
+    device = next(device for device in devices if device.duid == device_id)
+    product_info: dict[str, HomeDataProduct] = {product.id: product for product in home_data.products}
+    device_data = DeviceData(device, product_info[device.product_id].model)
+    mqtt_client = RoborockMqttClientV1(login_data.user_data, device_data)
+    networking = await mqtt_client.get_networking()
+    local_device_data = DeviceData(device, product_info[device.product_id].model, networking.ip)
+    local_client = RoborockLocalClientV1(local_device_data)
+    status = await local_client.get_status()
+    click.echo(json.dumps(status.as_dict(), indent=4))
+
+
 @click.command()
 @click.option("--device_id", required=True)
 @click.option("--cmd", required=True)
@@ -192,6 +259,9 @@ async def parser(_, local_key, device_ip, file):
 cli.add_command(login)
 cli.add_command(discover)
 cli.add_command(list_devices)
+cli.add_command(list_scenes)
+cli.add_command(execute_scene)
+cli.add_command(status)
 cli.add_command(command)
 cli.add_command(parser)
 
