@@ -13,7 +13,6 @@ from construct import (  # type: ignore
     Bytes,
     Checksum,
     ChecksumError,
-    Const,
     Construct,
     Container,
     GreedyBytes,
@@ -36,11 +35,18 @@ from roborock.roborock_message import RoborockMessage
 
 _LOGGER = logging.getLogger(__name__)
 SALT = b"TXdfu$jyZ#TZHsg4"
+A01_HASH = "726f626f726f636b2d67a6d6da"
 BROADCAST_TOKEN = b"qWKYcdQWrbm9hPqe"
 AP_CONFIG = 1
 SOCK_DISCOVERY = 2
 
 
+def md5hex(message: str) -> str:
+    md5 = hashlib.md5()
+    md5.update(message.encode())
+    return md5.hexdigest()
+
+
 class RoborockProtocol(asyncio.DatagramProtocol):
     def __init__(self, timeout: int = 5):
         self.timeout = timeout
@@ -199,12 +205,22 @@ class EncryptionAdapter(Construct):
 
         :param obj: JSON object to encrypt
         """
+        if context.version == b"A01":
+            iv = md5hex(format(context.random, "08x") + A01_HASH)[8:24]
+            decipher = AES.new(bytes(context.search("local_key"), "utf-8"), AES.MODE_CBC, bytes(iv, "utf-8"))
+            f = decipher.encrypt(obj)
+            return f
         token = self.token_func(context)
         encrypted = Utils.encrypt_ecb(obj, token)
         return encrypted
 
     def _decode(self, obj, context, _):
         """Decrypts the given payload with the token stored in the context."""
+        if context.version == b"A01":
+            iv = md5hex(format(context.random, "08x") + A01_HASH)[8:24]
+            decipher = AES.new(bytes(context.search("local_key"), "utf-8"), AES.MODE_CBC, bytes(iv, "utf-8"))
+            f = decipher.decrypt(obj)
+            return f
         token = self.token_func(context)
         decrypted = Utils.decrypt_ecb(obj, token)
         return decrypted
@@ -218,11 +234,8 @@ class OptionalChecksum(Checksum):
         hash2 = self.hashfunc(self.bytesfunc(context))
         if hash1 != hash2:
             raise ChecksumError(
-                "wrong checksum, read %r, computed %r"
-                % (
-                    hash1 if not isinstance(hash1, bytestringtype) else binascii.hexlify(hash1),
-                    hash2 if not isinstance(hash2, bytestringtype) else binascii.hexlify(hash2),
-                ),
+                f"wrong checksum, read {hash1 if not isinstance(hash1, bytestringtype) else binascii.hexlify(hash1)}, "
+                f"computed {hash2 if not isinstance(hash2, bytestringtype) else binascii.hexlify(hash2)}",
                 path=path,
             )
         return hash1
@@ -230,9 +243,9 @@ class OptionalChecksum(Checksum):
 
 class PrefixedStruct(Struct):
     def _parse(self, stream, context, path):
-        subcon1 = Peek(Optional(Const(b"1.0")))
+        subcon1 = Peek(Optional(Bytes(3)))
         peek_version = subcon1.parse_stream(stream, **context)
-        if peek_version is None:
+        if peek_version not in (b"1.0", b"A01"):
             subcon2 = Bytes(4)
             subcon2.parse_stream(stream, **context)
         return super()._parse(stream, context, path)
@@ -254,7 +267,7 @@ class PrefixedStruct(Struct):
 
 _Message = RawCopy(
     Struct(
-        "version" / Const(b"1.0"),
+        "version" / Bytes(3),
         "seq" / Int32ub,
         "random" / Int32ub,
         "timestamp" / Int32ub,
@@ -283,7 +296,7 @@ _BroadcastMessage = Struct(
     "message"
     / RawCopy(
         Struct(
-            "version" / Const(b"1.0"),
+            "version" / Bytes(3),
             "seq" / Int32ub,
             "protocol" / Int16ub,
             "payload" / EncryptionAdapter(lambda ctx: BROADCAST_TOKEN),
@@ -346,3 +359,56 @@ class _Parser:
 
 MessageParser: _Parser = _Parser(_Messages, True)
 BroadcastParser: _Parser = _Parser(_BroadcastMessage, False)
+
+
+Decoder = Callable[[bytes], list[RoborockMessage]]
+Encoder = Callable[[RoborockMessage], bytes]
+
+
+def create_mqtt_decoder(local_key: str) -> Decoder:
+    """Create a decoder for MQTT messages."""
+
+    def decode(data: bytes) -> list[RoborockMessage]:
+        """Parse the given data into Roborock messages."""
+        messages, _ = MessageParser.parse(data, local_key)
+        return messages
+
+    return decode
+
+
+def create_mqtt_encoder(local_key: str) -> Encoder:
+    """Create an encoder for MQTT messages."""
+
+    def encode(messages: RoborockMessage) -> bytes:
+        """Build the given Roborock messages into a byte string."""
+        return MessageParser.build(messages, local_key, prefixed=False)
+
+    return encode
+
+
+def create_local_decoder(local_key: str) -> Decoder:
+    """Create a decoder for local API messages."""
+
+    # This buffer is used to accumulate bytes until a complete message can be parsed.
+    # It is defined outside the decode function to maintain state across calls.
+    buffer: bytes = b""
+
+    def decode(bytes: bytes) -> list[RoborockMessage]:
+        """Parse the given data into Roborock messages."""
+        nonlocal buffer
+        buffer += bytes
+        parsed_messages, remaining = MessageParser.parse(buffer, local_key=local_key)
+        buffer = remaining
+        return parsed_messages
+
+    return decode
+
+
+def create_local_encoder(local_key: str) -> Encoder:
+    """Create an encoder for local API messages."""
+
+    def encode(message: RoborockMessage) -> bytes:
+        """Called when data is sent to the transport."""
+        return MessageParser.build(message, local_key=local_key)
+
+    return encode
